import requests
import warnings
import json
import re
import os

warnings.filterwarnings('ignore', message='Unverified HTTPS request')
authToken = None
timeout = None
verifySSL = False
baseurl = '/api/v1/'
authbaseurl = '/api/system/v1/'

class apic():
	def __init__(self, ip, uid, pw=None, **kwargs):
		global timeout
		global authToken
		self.ip = ip
		self.uid = uid
		self.pw = pw

		#Setting possible keyword adguments:
		try:
			self.port = kwargs['port']
		except:
			self.port = "443"
		
		try:
			self.verifySSL = False
			verifySSL = kwargs['verifySSL']
		except:
			self.verifySSL = False
			verifySSL = False
		try:
			self.authToken = kwargs['authToken']
			authToken = kwargs['authToken']
		except:
			pass

		if timeout == None:
			try:
				timeout = kwargs['timeout']
				self.timeout = timeout
			except:
				timeout = 5
				self.timeout = timeout
			else:
				pass

		if authToken == None:
			#Authenticate and retreive an authToken
			AuthURL = "https://" + self.ip + ":" + self.port + authbaseurl + "auth/token"
			payload = {}
			headers = {
			}
			try:
				print ("Authenticating...")
				response = requests.request("POST", AuthURL, headers=headers, data=payload, verify=verifySSL, auth=(self.uid, self.pw), timeout=timeout)
			except :
				print("Error: Timeout connection to DNA-C. Most likely a network reachability issue")
				exit()
			data = json.loads(response.text)
			if 'error' in data:
				print (data['error'])
				exit()
			else:
				print ("Login Success")
			authToken = (data['Token'])	
			self.authToken = authToken
		else:
			#print ("Reusing existing key..")
			self.authToken = authToken


	def taskStatus(self, **kwargs):
		try:
			self.taskId = kwargs['id']
		except:
			pass

		if self.taskId == None:
			raise ezDNACError('No previous task to check')
		
		url = "https://" + self.ip + ":" + self.port + "/api/v1/task/" + self.taskId
		payload = {}
		headers = {
		'x-auth-token': self.authToken,
		'Content-Type': 'application/json',
		}
		response = requests.request("GET", url, headers=headers, json=payload, verify=verifySSL, timeout=timeout)
		data = json.loads(response.text)
		return data['response']
		



	#Get the selected device ID from serial:
	def getAllDevices(self):
		url = "https://" + self.ip + ":" + self.port + baseurl + "network-device/"
		payload = {}
		headers = {
		'x-auth-token': self.authToken
		}
		response = requests.request("GET", url, headers=headers, data = payload, verify=verifySSL, timeout=timeout)
		data = json.loads(response.text)	
		return data

	def id_from_serial(self, serialNumber):
		switches = self.getAllDevices()
		for switch in switches['response']:
			if switch['serialNumber'] == serialNumber:
				switchId = switch['id']
		try:
			return switchId
		except KeyError:
			return None

	def getTemplateId(self, templateName):
		url = "https://" + self.ip + ":" + self.port + baseurl + "template-programmer/project"
		payload = {}
		headers = {
		'x-auth-token': self.authToken
		}
		response = requests.request("GET", url, headers=headers, data = payload, verify=verifySSL, timeout=timeout)
		data = json.loads(response.text)	

		for projects in data:
			for templates in projects['templates']:
				try:
					if templates['name'] == templateName:
						return (templates['id'])
				except:
					return None

	def getSites(self, **kwargs):
		try:
			searchsite=kwargs['site']
		except:
			searchsite=None
		if searchsite == None:
			url = "https://" + self.ip + ":" + self.port + "/dna/intent/api/v1/site"
		else:
			url = "https://" + self.ip + ":" + self.port + "/dna/intent/api/v1/site?name=" + searchsite + ""
		
		payload = {}
		headers = {
		'x-auth-token': self.authToken,
		'Content-Type': 'application/json',
		'__runsync': 'true',
		'__timeout': '10',
		'__persistbapioutput': 'true',
		}
		response = requests.request("GET", url, headers=headers, data=payload, verify=verifySSL, timeout=timeout)
		data = json.loads(response.text)

		if searchsite != None:		
			for site in data['response']:
				return site['id']

		return data['response']
		


	def getPnpDevices(self, **kwargs):
		try:
			serialNumber = kwargs['sn']
		except:
			serialNumber = None
		url = "https://" + self.ip + ":" + self.port + "/dna/intent/api/v1/onboarding/pnp-device?state=Unclaimed"
		payload = {}
		headers = {
		'x-auth-token': self.authToken
		}
		response = requests.request("GET", url, headers=headers, data=payload, verify=verifySSL, timeout=timeout)
		data = json.loads(response.text)

		if serialNumber == None:
			return data
		else:
			for device in data:
				if device['deviceInfo']['serialNumber'] == serialNumber:
					ret = device

			try:
				return device
			except:
				raise ezDNACError('Device with serial number' + serialNumber + ' not found in pnp.')



	def getInventoryDevies(self, **kwargs):
		try:
			serialNumber = kwargs['sn']
		except:
			serialNumber = None
		try:
			deviceId = kwargs['id']
		except:
			deviceId = None

		if deviceId != None:
			url = "https://" + self.ip + ":" + self.port + baseurl + "network-device/" + deviceId
		else:
			url = "https://" + self.ip + ":" + self.port + baseurl + "network-device/"

		payload = {}
		headers = {
		'x-auth-token': self.authToken
		}
		response = requests.request("GET", url, headers=headers, data = payload, verify=verifySSL, timeout=timeout)
		data = json.loads(response.text)	
		
		if deviceId != None:
			return data['response']
		else:
			if serialNumber == None:
				return data['response']
			else:
				for device in data['response']:
					if device['serialNumber'] == serialNumber:
						return device


	def pullTemplates(self, **kwargs):
		path = ""
		projectName = None
		try:
			projectName = kwargs['project']
		except:
			pass
		try:
			path = kwargs['path']
		except:
			pass

		url = "https://" + self.ip + ":" + self.port + baseurl + "template-programmer/project"
		headers = {
		  'x-auth-token': self.authToken,
		  'Content-Type': 'application/json',
		}
		response = requests.request("GET", url, headers=headers, verify=verifySSL)
		data = json.loads(response.text)
		templateslist = []
		#Get the id of interesting templates:
		for projects in data:
			if projectName != None:
				if (projects['name']) == projectName:
					for template in projects['templates']:
						templates = {}
						templates['id'] = template['id']
						templateslist.append(templates)
			else:
				for template in projects['templates']:
					templates = {}
					templates['id'] = template['id']
					templateslist.append(templates)
				
		for template in templateslist:
			url = "https://" + self.ip + ":" + self.port + baseurl + "template-programmer/template/"+template['id']
			headers = {
		  	'x-auth-token': self.authToken,
			'Content-Type': 'application/json',
			}
			response = requests.request("GET", url, headers=headers, verify=verifySSL)
			data = json.loads(response.text)
			filename = path + data['name'] + ".json"
			with open(filename, 'w') as out:
				out.write(str(json.dumps(data, indent=4)))

		if path == "":
			path = "local folder"
		if projectName != None:
			return "All templates in project: " + projectName + " are synced to: " + path		
		else:
			return "All templates in all projects are synced to: " + path		

	def pushTemplates(self, **kwargs):
		path = ""
		try:
			path = kwargs['path']
		except:
			pass

		#takes all the .json files from directory
		files = os.listdir(path)
		for file in files:
			if not re.match(r'.*.json', file):
				files.remove(file)
	

		#Firest check if template already exists:
		for file in files:
			with open(path+file) as templateData:
				templateFile = json.load(templateData)
				templateName = templateFile['name']
				projectName = templateFile['projectName']
				
				#Check whats already existing, based on project/template tree name
				url = "https://" + self.ip + ":" + self.port + baseurl + "template-programmer/project/"
				payload = {}
				headers = {
			  	'x-auth-token': self.authToken,
				'Content-Type': 'application/json',
				}
				response = requests.request("GET", url, headers=headers, data = payload, verify=verifySSL)
				data = json.loads(response.text)
				templateExists = False
				projectExists = False
				for project in data:
					if project['name'] == projectName:
						projectExists = True
						projectId = project['id']
						print ("project exists: " + projectName + " - " + projectId)

#						for templates in project['templates']:
#							if templates['id']						

						for templates in project['templates']:
							if templates['name'] == templateName:
								templateId = templates['id']
								templateExists = True
								print ("template finns: " + templateName + " - " + templateId)


				if projectExists == False:
					print ("Creating missing project: " + projectName)
					url = "https://" + self.ip + ":" + self.port + "/dna/intent/api/v1/template-programmer/project"

					payload = {
						  "name": projectName,
							}
					headers = {
				  	'x-auth-token': self.authToken,
					'Content-Type': 'application/json',
					}
					response = requests.request("POST", url, headers=headers, json=payload, verify=verifySSL)
					data = json.loads(response.text)
					
					#Since the project didn't exist, we need to fetch it's new id. 
					taskId = data['response']['taskId']
					datafromtask = self.taskStatus(id=taskId)					
					projectId = datafromtask['data']

				if templateExists == False:
					print ("Creating missing template: "+ templateName)
					url = "https://" + self.ip + ":" + self.port + "/dna/intent/api/v1/template-programmer/project/" + projectId +"/template"
					payload = templateFile				
					#Remove keys, making the payload suitable for new-creation of template. Hence removing id etc.
					if 'id' in payload:
						del payload['id']
					for param in payload['templateParams']:
						if 'id' in param:
							del param['id']					
					headers = {
				  	'x-auth-token': self.authToken,
					'Content-Type': 'application/json',
					}
					response = requests.request("POST", url, headers=headers, json=payload, verify=verifySSL)
					data = json.loads(response.text)
					try:
						self.taskId = data['response']['taskId']
					except:
						self.taskId = None

				if templateExists == True:
				#Since the template exists, PUT the file from directory to make sure the active template is same version.
					print ("Template " + templateName + " already exists, updating.")
					url = "https://" + self.ip + ":" + self.port + baseurl + "template-programmer/template/"
					payload = templateFile
					payload['id'] = templateId

					for params in payload['templateParams']:
						del params['id']

					headers = {
				  	'x-auth-token': self.authToken,
					'Content-Type': 'application/json',
					}
					response = requests.request("PUT", url, headers=headers, json=payload, verify=verifySSL)
					data = json.loads(response.text)



#When initialized, populate device parameters:
#Retreive switchId based on serialnumber
class device():
	def __init__(self, dna, **kwargs):
		self.authToken = dna.authToken
		self.dnacIP = dna.ip
		self.port = dna.port
		self.uid = dna.uid
		self.pw = dna.pw
		self.state = ""
		self.dnac = apic(self.dnacIP, self.uid, pw=self.pw)
		try:
			self.id = kwargs['id']
			self.initMethod = 'id'
		except:
			self.serialNumber = kwargs['sn']
			self.initMethod = 'sn'

		#if init method is id, the device must be in inventory. Populate all attributes:		
		if self.initMethod == 'id':
			try:
				self.state 			= "Provisioned"
				self.serialNumber 	= self.dnac.getInventoryDevies(id=self.id)['serialNumber']
				self.hostname		= self.dnac.getInventoryDevies(id=self.id)['hostname']
				self.platform		= self.dnac.getInventoryDevies(id=self.id)['platformId']
				self.softwareVersion= self.dnac.getInventoryDevies(id=self.id)['softwareVersion']
				self.softwareType	= self.dnac.getInventoryDevies(id=self.id)['softwareType']
			except:
				raise ezDNACError('device with id not found')
		#if method is sn, the device can be either in inventory or pnp, have to try both
		elif self.initMethod == 'sn':
			#Try populate the attributes via the inventory:
			try:
				INVdevices 			= self.dnac.getInventoryDevies(sn=self.serialNumber)
				self.id 			= INVdevices['id']
				self.hostname		= INVdevices['hostname']
				self.platform		= INVdevices['platformId']
				self.softwareVersion= INVdevices['softwareVersion']
				self.softwareType	= INVdevices['softwareType']
				self.state 			= "Provisioned"
			except:
				pass
			if self.state != "Provisioned":
				try:
				#Otherwise try populate attributes via pnp:
					PNPdevices 			= self.dnac.getPnpDevices(sn=self.serialNumber)
					self.id 			= PNPdevices['id']
					self.state  		= PNPdevices['deviceInfo']['state']
					self.hostname		= PNPdevices['deviceInfo']['name']
					self.platform		= PNPdevices['deviceInfo']['pid']
					self.softwareVersion= PNPdevices['deviceInfo']['imageVersion']
					self.softwareType	= PNPdevices['deviceInfo']['agentType'] 
				except:
					raise ezDNACError('device with serialNumber' + self.serialNumber +' not found')

	def getTopology(self):
		ret = []
		url = "https://" + self.dnac.ip + ":" + self.dnac.port + baseurl + "topology/physical-topology/"
		payload = {}
		headers = {
		'x-auth-token': self.authToken
		}
		
		response = requests.request("GET", url, headers=headers, data = payload, verify=verifySSL, timeout=timeout)
		data = json.loads(response.text)
		connections = {}
		links = []
		for link in data['response']['links']:
		
			try:
				connections['sourcenode'] = link['source']
				connections['remotenode'] = link['target']
				connections['sourceif'] = link['startPortName']
				connections['remoteif'] = link['endPortName']
				links.append(dict(connections))
			except:
				pass
		ret = links
		return ret


	def getConnections(self):
		ret = []
		url = "https://" + self.dnac.ip + ":" + self.dnac.port + baseurl + "topology/physical-topology/"
		payload = {}
		headers = {
		'x-auth-token': self.authToken
		}
		response = requests.request("GET", url, headers=headers, data = payload, verify=verifySSL)
		data = json.loads(response.text)
		connections = {}
		links = []
		for link in data['response']['links']:
			try:
				if link['source'] == self.id:
					print ("source")
					connections['remotenode'] = link['target']
					connections['remoteif'] = link['endPortName']
					connections['localif'] = link['startPortName']
					links.append(dict(connections))
				elif link['target'] == self.id:
					print ("arget")
					connections['remotenode'] = link['source']
					connections['remoteif'] = link['startPortName']
					connections['localif'] = link['endPortName']
					links.append(dict(connections))
			except:
				pass
		ret = links
		return ret


	def deployTemplate(self, templateId, templateParams):
			url = "https://" + self.dnac.ip + ":" + self.dnac.port + baseurl + "template-programmer/template/deploy"
			payload = {
			  "templateId": templateId,
			   "targetInfo": [
			     {
			      "id": self.id,
			      "type": "MANAGED_DEVICE_UUID",
				  "params": templateParams
			     }
				]}
			
			headers = {
			  'x-auth-token': self.authToken,
			  'Content-Type': 'application/json',
			}
			response = requests.request("POST", url, headers=headers, json=payload, verify=verifySSL, timeout=10)
			if response.status_code == 202:
				try:
					#Försök nyckla ut deploymentId som respons. API responsen är trasig så får köra regex
					result = json.loads(response.text)['deploymentId']
					deploymentId = (str(re.findall(r'Template Deployemnt Id.*', result)).strip("['Template Deployemnt Id: ]"))
					if deploymentId == "":
						self.deploymentId = None
						return None
					else:
						self.deploymentId = deploymentId
						return {"deploymentId":deploymentId}
				except:
					return None

	def deployTemplateStatus(self, **kwargs):
		try:
			self.deploymentId = kwargs['id']
		except:
			pass
		url = "https://" + self.dnac.ip + ":" + self.dnac.port + "/dna/intent/api/v1/template-programmer/template/deploy/status/" + self.deploymentId
		payload = {}
		headers = {
		'x-auth-token': self.authToken,
		'Content-Type': 'application/json',
		}
		response = requests.request("GET", url, headers=headers, json=payload, verify=verifySSL, timeout=timeout)

		data = json.loads(response.text)
		return data['status']



	def findNextPortchannel(self):
		url = "https://" + self.dnac.ip + ":" + self.dnac.port + baseurl + "interface/network-device/" + self.id
		payload = {}
		headers = {
			  'x-auth-token': self.authToken,
			  'Content-Type': 'application/json',
			}
		response = requests.request("GET", url, headers=headers, json=payload, verify=verifySSL, timeout=5)
		config = json.loads(response.text)

		existing_ids = []
		for interface in config['response']:
			if re.match(r'Port-channel.*', str(interface['portName'])):
				intf = int(str(interface['portName']).strip("'Port-channel"))
				existing_ids.append(intf)
					
		for i in range(1,49):
			if (i) not in existing_ids:
				next_id = i
				break
		return next_id
		
	def assignToSite(self, siteId):
		url = "https://" + self.dnac.ip + ":" + self.port + "/dna/system/api/v1/site/" + siteId + "/device"
		payload = {
		  "device": [
		    {
		      "ip": self.ip
		    }
		  ]
		}
		print (payload)
		headers = {
		'x-auth-token': self.authToken,
		'Content-Type': 'application/json',
		'__runsync': 'true',
		'__timeout': '10',
		'__persistbapioutput': 'true',
		}
		response = requests.request("POST", url, headers=headers, json=payload, verify=verifySSL, timeout=timeout)
		#data = json.loads(response.text)
		return response.text



	def getNeighbors(self):
		connections = self.getConnections()
		neighbors = []
		for link in connections:
			if link['remotenode'] in neighbors:
				continue
			else:
				neighbors.append(link['remotenode'])
		return neighbors


	#return every interface connected to us from specific neighbor
	def getNeighborIfs(self, neighbor):
		connections = self.getConnections()
		interfaces = []
		for link in connections:
			if link['remotenode'] == neighbor:
				interfaces.append(link['remoteif'])
		return interfaces


	def getModules(self):
		url = "https://" + self.dnac.ip + ":" + self.port + baseurl + "network-device/module?deviceId=" + self.id
		payload = {}
		headers = {
		'x-auth-token': authToken
		}
		response = requests.request("GET", url, headers=headers, data = payload, verify=verifySSL)
		data = json.loads(response.text)
		modules = data['response']

		self.modules = modules
		
		switches = []
		for module in modules:
				name = module['name']
				switch = str((re.findall(r'Switch \d', name))).strip("[']")
				switches.append(switch)
		self.stackcount = len((set(switches)))
		
		return modules

	def claimDevice(self, siteId, **kwargs):
		url = "https://" + self.dnac.ip + ":" + self.port + "/api/v1/onboarding/pnp-device/site-claim"
		try:
			payload = {
		    "siteId": siteId,
		     "deviceId": pnpDeviceId,
		     "type": "Default",
		     "imageInfo": {"imageId": "None", "skip": true},
		     "configInfo": {"configId": kwargs['templateId'], "configParameters":[kwargs[params]]}
			}
		except:
			payload = {
	        "siteId": siteId,
	         "deviceId": self.id,
	         "type": "Default",
	         "imageInfo": {"imageId": "None", "skip": "true"},
	         "configInfo": {"configId": "", "configParameters":[]}
			}

		headers = {
		'x-auth-token': authToken,
		'Content-Type': 'application/json',
		}
		response = requests.request("POST", url, headers=headers, json=payload, verify=verifySSL, timeout=timeout)
		data = json.loads(response.text)
		return data



class ezDNACError(Exception):
    pass



#####################################################
## Written by Johan Lahti, CCIE60702                #
## https://github.com/johan-lahti                   #
## shiproute.net                                    #
#####################################################



from ezdnac import device, apic


#####################################################################################################################
## This file is holding some examples of how to use the ezdnac.py 
##
####Written by Johan Lahti, CCIE60702, Conscia Sweden.###############################################################
#####################################################################################################################

####################################################################################################################
## The driver behind the creation of ezdnac (easy dna-c) is to make it easier to script towards its API. 
## Some abstraction is done, for example the management of authorization. The module will automatically 
## use the auth-api to athenticate and autorize and after that reuse the authToken for the rest of the script runtime
##
## ezdnac.py defines two classes, one for the DNAC itself and its parameters. 
## and one for the device/devices you want to manage. 
## The apic class is holding all functions that manages dnac-wide stuff such as authentication, topology and
## templates.
##
## The device holds all functions that is necessary to populate a device, some parts are automatically always 
## populated using the apic class such as hostname, type etc. 
##
###################################    APIC and Device    #######################################################
#################################################################################################################
## The device class also holds operational functions, such as template deployment for a given device.
## 
## First example shows how to initalize a dna-c apic, using the apic class. The init needs following inputs:
## 1. ip address of the dna-c
## 2. username
## 3. password
## -port (optional, unless specified port 443 will be used.)
## -timeout (optinal, default 5sek)
## Second example shows how to initialize an object for a specific device, 
## enter serialnumber or deviceId and apic-object. 
## 
## In the example we initialize two nodes device1 and device2.
###################################################################################################################
## Example 1: 
#dna_c = apic('10.10.100.40','admin', 'password, timeout=2')
##
## Example 2:
#device1 = device(sn='ABCDEF12345', dna_c)
#device2 = device(id='1111111-aaaa-2222-bbbb-333333333333', dna_c)
#
###################################    Device    ##################################################################
###################################################################################################################
## Now that the objects are created, you can refer to its' 
## defined attributes such as hostname and softwareType 
## 
####################################################################################################################
## Example: 
#print (device1.hostname)
#print (device1.softwareType + " ver: " + device1.softwareVersion)
#print (device2.hostname)
#print (device2.platform)
#
#####################################################################################################################
####################################################################################################################
## With the function getTopology in the device class, it will return a dictionary 
## with all links for the dnac that the device is claimed to. Note that this includes the whole
## Topology as known to the dnac.
##  
## 
####################################################################################################################
## Example: 
# all_connections = device1.getTopology()
#
####################################################################################################################
####################################################################################################################
## The function getSites gives you a dictionary of all sites. IF you specify site=sitename in the input that site 
## is returned only. Sitename should include the whole siteNameHierarchy, ex "Global/Area/sitename"
## 
####################################################################################################################
####################################################################################################################
## With the function getConnections, you will get all connections for the specified device.
##  You can use keys such as 'remotenode' or 'localif'. 
## The keys prefixed with 'remote' will always refer to the other side from the specified switches' perspecitve
##  In the example below we retreive all connections to device1
##
####################################################################################################################
## Example: 
#device_connections = device1.getConnections()
#
#
#print (" - Displaying device connections - ")
#
#for connection in device_connections:
#	print ("Device is connected to " + connection['remotenode'])
#	print ("Remote interface: " + connection['remoteif'])
#	print ("Local interface: " + connection['localif'])
#
#
###################################################################################################################
###################################################################################################################
##To deploy a template following steps is necessary:
## 1. define a dictionary containing all template variables
## 2. append the template id to the dictionary
## 3. use the function deployTemplate in the device class with the deviceId and params-dictionary as input
##
## To translate template name to template id, use the function getTemplateid in the
## apic class.
##
## Example showing the workflow. 
## 1. initialize the dnac
## 2. translate template name to template id
## 3. initialize the device to deploy the template to 
## 4. define the template input dictionary
## 5. run the deploy
##
####################################################################################################################
####################################################################################################################
## Another handy function is findNextPortchannel under the device class. It discovers
## the next configurable port-channel id for the device. Let's say id 1,4 and 8 are configured
## already, it will return the first free id which would be 2 in that case.
## 
####################################################################################################################
## Example: 
#apic_sweden = apic('ip address of dna-c','username', 'password')
#templateId = (apic_sweden.getTemplateid('template_name'))
#device1 = device('ABCDEF12345', dna_c)
#
#params = {
#"description": 'STuffz',
#"interface": 'Gi1/0/2'
#}
#
#device1.deployTemplate(templateId,dict(params))

###################################    APIC      ################################################################
#################################################################################################################
## Here comes some apic specific features. The first one written was the management of templates. 
## The functions are named pullTemplates and pushTemplates. And are very easy to use. Input parameters area
## for the pullTemplates: 
##  -  path (optional - directory where the templates shall be stored (default is local path))
##  -  project (optional, if only a specific project is to be pulled. Default is all projects)
##
####################################################################################################################
#
# #This line gets all templates from 'Day1' project and stores them in the folder 'templates'
#print (dna_c.pullTemplates(project='Day1', path='templates/'))
#
# #This line pushes all templates in the given folder to the DNA-C.
#
# print (dna_c.pushTemplates(path=')'templates/')
####################################################################################################################
## The project/template tree matches on it's name. The idea is to enable sync between more than one DNA-C, hence matchin
## on the id is not appropriate. It matches on the whole tree, meaning if the template name exists under another project
## it is still going to be seen as a unique template. 
## When pushing a template, the function will do multiple steps:
## 1. Check if the project exists.
## 2. Check if the template exists.
## If the project does not exist = create project and create template
## If the project exists but not the template = create the tempalte within the project.
## If the template exists = Update it according to the local file. 
## This enables offline editing as well as syncinc templates and storing them elsewhere like GIT.
####################################################################################################################
####################################################################################################################









import click
import ezdnac
import os
import tabulate
import json

settings_file = ".ezdnac_cli_settings.py"
try:
	with open(settings_file) as file:
			settings = json.load(file)
except:
	DNA_IP = None
	DNA_USER = None
	AUTHTOKEN = None
try:
	DNA_IP = settings['DNA_IP'] 
	DNA_USER = settings['DNA_USER']
	AUTHTOKEN = settings['authToken']
except:
	pass

def initDnac():
	global dnac
	if DNA_IP == None or DNA_USER == None or AUTHTOKEN == None:
		print ("Login details not set. Use login command to login first")
		exit(1)
	else:
		dnac = ezdnac.apic(DNA_IP, DNA_USER, authToken=AUTHTOKEN)


@click.group()
def cli():
    """Command line tool for deploying templates to CISCO DNA-C.
    """
    pass

@click.command()
@click.option("--ip")

def login(ip):
	DNA_USER = click.prompt("username")	
	DNA_PW = click.prompt("password", hide_input=True)
	settings = {}
	settings['DNA_IP'] = ip
	settings['DNA_USER'] = DNA_USER

	if ip == None:
		DNA_IP = click.prompt("DNA-C IP")

	dnac = ezdnac.apic(DNA_IP, DNA_USER, pw=DNA_PW)
	settings['authToken'] = dnac.authToken
	settings['DNA_IP'] = DNA_IP
	settings['DNA_USER'] = DNA_USER
	
	with open(settings_file, 'w+') as out:
		out.write(str(json.dumps(settings)))
	click.secho("Credentials set for user" + DNA_USER)

@click.group()
def show():
	initDnac()
	pass

@show.command()
def pnpq():
	initDnac()

	with open(settings_file) as file:
		settings = json.load(file)
	try:
		DNA_IP = settings['DNA_IP']
		authToken = settings['authToken']
	except:
		click.secho("Credentials not set or expired. Use ezdnac login to reset.")
	click.secho("Retrieving the devices in PnP queue.")
		
	deviceList = []
	devices = dnac.getPnpDevices()

	for device in devices:
		dev = device['deviceInfo']
		deviceData = dev['serialNumber'] , dev['pid'], dev['state']	
		try:
			deviceDat2 = (*deviceData, dev['macAddress'])
		except:
			deviceDat2 = (*deviceData, "00:00:00:00:00:00")

		deviceList.append(deviceDat2)

	headers = ["Serial Number", "Device Type", "State", "Mac"]
	click.echo(tabulate.tabulate(deviceList, headers, tablefmt="fancy_grid"))


@show.command()
def inventory():
	initDnac()
	data = dnac.getAllDevices()
	
	deviceList = []
	for device in data['response']:
		hostname = device['hostname']
		serialNumber = device['serialNumber']
		mac = device['macAddress']
		ip = device['managementIpAddress']
		pid = device['platformId']
		deviceData = [hostname, ip, mac, pid, serialNumber]
		deviceList.append(deviceData)	
	
	headers = ['Hostname', "MGMT IP", "MAC", "Pid", "Serial Number"]
	click.echo(tabulate.tabulate(deviceList, headers, tablefmt="fancy_grid"))	


@show.group()
def device():
	pass

@device.command()
def neighbors():
	initDnac()
	click.echo("show neighbors...")
	serialNumber =click.prompt("serialNumber")

	device = ezdnac.device(dnac, sn=serialNumber)
	click.secho("Retrieving the neighbors for serial: " + serialNumber)

	neighbors = device.getTopology()

	topo = []
	if len(neighbors) == 0:
		print ("No neighbors!")
	else:
		for neighbor in neighbors:
			neihgId = neighbor['remotenode']
			neighIntf = neighbor['remoteif']
			localIntf = neighbor['sourceif']
			neig = ezdnac.device(dnac, id=neihgId)
			
			linkData = [localIntf, neig.hostname, neighIntf]

			topo.append(linkData)
			
	headers = ['Local Interface', 'Connected to hostname', 'Neighbor Interface']
	click.echo(tabulate.tabulate(topo, headers, tablefmt="fancy_grid"))

@device.command()
def details():
	initDnac()
	click.echo("Show device details...")

cli.add_command(device)
cli.add_command(show)
cli.add_command(neighbors)
cli.add_command(details)
cli.add_command(pnpq)
cli.add_command(login)

if __name__ == "__main__":
    cli()